# Copyright 2021 VMware, Inc.
# SPDX-License-Identifier: Apache-2.0
import logging
import os
import pathlib
from typing import Dict

from taurus.api.job_input import IJobInput

log = logging.getLogger(__name__)


class CsvIngester:
    def __init__(self, job_input: IJobInput):
        self.__job_input = job_input

    def ingest(self, input_file: pathlib.Path, destination_table: str, options: Dict):
        import pandas as pd

        df = pd.read_csv(str(input_file), **options)
        df.dropna(how="all", inplace=True)

        self.__job_input.send_tabular_data_for_ingestion(
            rows=df.values,
            column_names=df.columns.values.tolist(),
            destination_table=destination_table,
        )
        log.info(
            f"Ingested data from {input_file} into table {destination_table} successfully."
        )


def run(job_input: IJobInput) -> None:
    csv_file = pathlib.Path(job_input.get_arguments().get("file"))
    destination_table = job_input.get_arguments().get("destination_table", None)
    if not destination_table:
        destination_table = os.path.splitext(csv_file.name)[0]
    options = job_input.get_arguments().get("options", {})

    csv = CsvIngester(job_input)
    csv.ingest(csv_file, destination_table, options)



# Copyright 2021 VMware, Inc.
# SPDX-License-Identifier: Apache-2.0
"""
VDK CSV plugin script.
"""
import json
import logging
import os

import click
from click import Context
from taurus.api.plugin.hook_markers import hookimpl
from taurus.vdk.builtin_plugins.run.cli_run import run
from taurus.vdk.csv_ingest_job import csv_ingest_step

log = logging.getLogger(__name__)


@click.command(
    name="ingest-csv",
    help="Ingest CSV file."
    """
The ingestion destination depends on how vdk has been configured.
See vdk config-help  - search for "ingest" to check for possible ingestion configurations.

Examples:

\b
# This will ingest a CSV file into table with the same name as the csv file
# revenue.csv will be ingested into table revenue.
# Column names are inferred from the top row of the csv file.
vdkcli ingest-csv -f revenue.csv

\b
# This will ingest a CSV file into table passed as argument
vdkcli ingest-csv -f revenue.csv --table-name my_revenue_table

\b
# This will ingest a CSV file
# But we will switch the delimiter to tab instead of comma
# Effectively ingesting TSV (tab-separate) file.
vdkcli ingest-csv -f revenue.tsv --options='{"sep": "\\t"}'

\b
# This will ingest a CSV file.
# We will use custom column names.
vdkcli ingest-csv -f revenue.csv --options='{"names": ["gender", "os", "visits", "age", "revenue"]}'

               """,
    no_args_is_help=True,
)
@click.option(
    "-f",
    "--file",
    help="Path to the csv file. It must contain properly formatted csv file.",
    type=click.Path(exists=True, file_okay=True, dir_okay=False, resolve_path=True),
)
@click.option(
    "-t",
    "--table-name",
    default=None,
    type=click.STRING,
    help="The table in which the csv file will be ingested into."
    "If not specified, it will default to the csv file name without the extension",
)
@click.option(
    "-o",
    "--options",
    default="{}",
    type=click.STRING,
    help="""Pass extra options when reading CSV file formatted as json. For example {"sep": ";", "verbose": true} """
    "Those are the same options as passed to pandas.read_csv. See "
    "https://pandas.pydata.org/pandas-docs/stable/reference/api/pandas.read_csv.html.",
)
@click.pass_context
def ingest_csv(ctx: Context, file: str, table_name: str, options: str) -> None:
    args = dict(file=file, destination_table=table_name, options=json.loads(options))
    ctx.invoke(
        run,
        data_job_directory=os.path.dirname(csv_ingest_step.__file__),
        arguments=json.dumps(args),
    )
    click.echo(f"Ingesting csv file {file} finished.")


@hookimpl
def vdk_command_line(root_command: click.Group) -> None:
    root_command.add_command(ingest_csv)
